/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **  
 ** Permission is hereby granted, free of charge, to any person or organization 
 ** obtaining a copy of the software and accompanying documentation (the 
 ** "Software") to use, reproduce, display, distribute, sub-license, execute, 
 ** and transmit the Software, and to prepare derivative works of the Software, 
 ** and to permit third-parties to whom the Software is furnished to do so, in 
 ** accordance with:
 ** 
 ** (1) if the Software is obtained from Blackmagic Design, the End User License 
 ** Agreement for the Software Development Kit (“EULA”) available at 
 ** https://www.blackmagicdesign.com/EULA/DeckLinkSDK; or
 ** 
 ** (2) if the Software is obtained from any third party, such licensing terms 
 ** as notified by that third party,
 ** 
 ** and all subject to the following:
 ** 
 ** (3) the copyright notices in the Software and this entire statement, 
 ** including the above license grant, this restriction and the following 
 ** disclaimer, must be included in all copies of the Software, in whole or in 
 ** part, and all derivative works of the Software, unless such copies or 
 ** derivative works are solely in the form of machine-executable object code 
 ** generated by a source language processor.
 ** 
 ** (4) THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 ** OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT 
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE 
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, 
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 ** DEALINGS IN THE SOFTWARE.
 ** 
 ** A copy of the Software is available free of charge at 
 ** https://www.blackmagicdesign.com/desktopvideo_sdk under the EULA.
 ** 
 ** -LICENSE-END-
 */

#import <Cocoa/Cocoa.h>
#import <memory>
#import <string>
#import "DeckLinkDeviceDiscovery.h"
#import "DeckLinkDeviceState.h"
#import "CocoaViewController.h"
#import "com_ptr.h"

@interface AppDelegate : NSObject <NSApplicationDelegate> {

	// GUI
	IBOutlet NSWindow*		m_window;

	// Preview
	IBOutlet NSView*		m_previewView;

	// Playback
	IBOutlet NSTextField*	m_timeElapsed;
	IBOutlet NSSlider*		m_streamTimeSlider;
	IBOutlet NSTextField*	m_timeRemaining;
	IBOutlet NSPopUpButton*	m_deviceList;
	IBOutlet NSTextField*	m_displayModeName;
	IBOutlet NSButton*		m_openButton;
	IBOutlet NSButton*		m_playButton;
	IBOutlet NSPathControl*	m_playbackFilePath;

	std::string				m_filePath;

	intptr_t				m_selectedDevice;

	com_ptr<IDeckLinkScreenPreviewCallback>		m_screenPreviewCallback;

	std::unique_ptr<DeckLinkDeviceDiscovery>	m_deviceDiscovery;
	std::shared_ptr<DeviceManager>				m_deviceManager;
	std::unique_ptr<CocoaViewController>		m_viewController;

	// Media IO
	std::shared_ptr<DeckLinkMediaReader>		m_mediaReader;

	DeviceIOState								m_deviceState;
}

// UI Controls
- (IBAction)deviceSelected:(id)sender;
- (IBAction)openClicked:(id)sender;
- (IBAction)playClicked:(id)sender;
- (IBAction)playPositionChanged:(id)sender;

// Close the app when the main window is closed
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication*)app;

// Device arrival/removal
- (void)updateDevice:(NSInteger)deviceID enabled:(BOOL)enabled;
- (void)addDevice:(NSInteger)deviceID displayName:(CFStringRef)displayName enabled:(BOOL)enabled;
- (void)removeDevice:(NSInteger)deviceID;
- (void)clearDisplayModeName;
- (void)setDisplayModeName:(NSInteger)displayMode displayModeName:(CFStringRef)displayModeName;

// Device status
- (void)setFilePath:(const std::string&)filePath;

// Playback status
- (void)setPlayPosition:(double)position elapsedSeconds:(int64_t)elapsedSeconds totalSeconds:(int64_t)totalSeconds;

// UI state
- (void)showError:(CFStringRef)title detail:(CFStringRef)detail;
- (void)enableUI:(BOOL)enabled;
- (void)setDeviceState:(DeviceIOState)deviceState;

- (com_ptr<IDeckLinkScreenPreviewCallback>)screenPreviewCallback;

@end
